{% load static %}
{% load i18n %}
{% load lists %}
{% load codenerix_common %}
{% comment %}

django-codenerix

Codenerix GNU

Project URL : http://www.codenerix.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

{% endcomment %}

{% block angular_app_libraries %}
    <script type="text/javascript" src="{% static "codenerix/js/app.js" %}"></script>
    <script type="text/javascript" src="{% static "codenerix/js/controllers.js" %}"></script>
    <script type="text/javascript" src="{{ static_filters_row }}"></script>
    {% block angular_app_service %}
        <script type="text/javascript">
            var ws_entry_point="{{ws_entry_point}}";
            var static_partial_row="{{static_partial_row}}";
            var static_partial_header="{{static_partial_header}}";
            var static_partial_summary="{{static_partial_summary}}";
            angular.module('gestdocumentServices', ['ngResource'])
            .factory('Register', ['$resource', function($resource){
                return $resource('/{{ws_entry_point}}/:pk/:action', { json:'{}', pk:null, action:null },{
                    query:{ method: "GET", params: { json:'@json' }, isArray: false },
                    get:{ method: "GET", params: { pk:'@pk', action:'@action' }, isArray: false },
                }); 
            }]);
        </script>
    {% endblock %}
{% endblock %}

{% with langcode=LANGUAGE_CODE|default:"en" %}
    <div class="row clearfix">
    <div class="modal-body">

    </div>
    </div>
{% endwith %}
